package com.sun.xml.internal.messaging.saaj.soap;

import javax.activation.CommandInfo;
import javax.activation.MailcapCommandMap;
import javax.activation.CommandMap;
import com.sun.xml.internal.messaging.saaj.packaging.mime.util.ASCIIUtility;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.InternetHeaders;
import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.MimeUtility;
import javax.xml.soap.MimeHeader;
import java.io.OutputStream;
import java.io.InputStream;
import javax.activation.DataSource;
import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.MimePartDataSource;
import javax.xml.soap.SOAPException;
import java.io.IOException;
import com.sun.xml.internal.messaging.saaj.SOAPExceptionImpl;
import java.util.logging.Level;
import com.sun.xml.internal.messaging.saaj.util.ByteOutputStream;
import java.util.Iterator;
import java.util.List;
import com.sun.xml.internal.org.jvnet.mimepull.Header;
import com.sun.xml.internal.org.jvnet.mimepull.MIMEPart;
import javax.activation.DataHandler;
import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.MimeBodyPart;
import javax.xml.soap.MimeHeaders;
import java.util.logging.Logger;
import javax.xml.soap.AttachmentPart;

public class AttachmentPartImpl extends AttachmentPart
{
    protected static final Logger log;
    private final MimeHeaders headers;
    private MimeBodyPart rawContent;
    private DataHandler dataHandler;
    private MIMEPart mimePart;
    
    public AttachmentPartImpl() {
        this.rawContent = null;
        this.dataHandler = null;
        this.mimePart = null;
        this.headers = new MimeHeaders();
        initializeJavaActivationHandlers();
    }
    
    public AttachmentPartImpl(MIMEPart part) {
        this.rawContent = null;
        this.dataHandler = null;
        this.mimePart = null;
        this.headers = new MimeHeaders();
        this.mimePart = part;
        List<? extends Header> hdrs = part.getAllHeaders();
        for (Header hd : hdrs) {
            this.headers.addHeader(hd.getName(), hd.getValue());
        }
    }
    
    public int getSize() throws SOAPException {
        if (this.mimePart != null) {
            try {
                return this.mimePart.read().available();
            }
            catch (IOException e) {
                return -1;
            }
        }
        else {
            if (this.rawContent == null && this.dataHandler == null) {
                return 0;
            }
            if (this.rawContent != null) {
                try {
                    return this.rawContent.getSize();
                }
                catch (Exception ex2) {
                    AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0573.soap.attachment.getrawbytes.ioexception", new String[] { ex2.getLocalizedMessage() });
                    throw new SOAPExceptionImpl("Raw InputStream Error: " + ex2);
                }
            }
            else {
                ByteOutputStream bout = new ByteOutputStream();
                try {
                    this.dataHandler.writeTo(bout);
                }
                catch (IOException ex) {
                    AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0501.soap.data.handler.err", new String[] { ex.getLocalizedMessage() });
                    throw new SOAPExceptionImpl("Data handler error: " + ex);
                }
                return bout.size();
            }
        }
    }
    
    public void clearContent() {
        if (this.mimePart != null) {
            this.mimePart.close();
            this.mimePart = null;
        }
        this.dataHandler = null;
        this.rawContent = null;
    }
    
    public Object getContent() throws SOAPException {
        try {
            if (this.mimePart != null) {
                return this.mimePart.read();
            }
            if (this.dataHandler != null) {
                return this.getDataHandler().getContent();
            }
            if (this.rawContent != null) {
                return this.rawContent.getContent();
            }
            AttachmentPartImpl.log.severe("SAAJ0572.soap.no.content.for.attachment");
            throw new SOAPExceptionImpl("No data handler/content associated with this attachment");
        }
        catch (Exception ex) {
            AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0575.soap.attachment.getcontent.exception", ex);
            throw new SOAPExceptionImpl(ex.getLocalizedMessage());
        }
    }
    
    public void setContent(Object object, String contentType) throws IllegalArgumentException {
        if (this.mimePart != null) {
            this.mimePart.close();
            this.mimePart = null;
        }
        DataHandler dh = new DataHandler(object, contentType);
        this.setDataHandler(dh);
    }
    
    public DataHandler getDataHandler() throws SOAPException {
        if (this.mimePart != null) {
            return new DataHandler(new DataSource() {
                public InputStream getInputStream() throws IOException {
                    return AttachmentPartImpl.this.mimePart.read();
                }
                
                public OutputStream getOutputStream() throws IOException {
                    throw new UnsupportedOperationException("getOutputStream cannot be supported : You have enabled LazyAttachments Option");
                }
                
                public String getContentType() {
                    return AttachmentPartImpl.this.mimePart.getContentType();
                }
                
                public String getName() {
                    return "MIMEPart Wrapper DataSource";
                }
            });
        }
        if (this.dataHandler == null) {
            if (this.rawContent != null) {
                return new DataHandler(new MimePartDataSource(this.rawContent));
            }
            AttachmentPartImpl.log.severe("SAAJ0502.soap.no.handler.for.attachment");
            throw new SOAPExceptionImpl("No data handler associated with this attachment");
        }
        else {
            return this.dataHandler;
        }
    }
    
    public void setDataHandler(DataHandler dataHandler) throws IllegalArgumentException {
        if (this.mimePart != null) {
            this.mimePart.close();
            this.mimePart = null;
        }
        if (dataHandler == null) {
            AttachmentPartImpl.log.severe("SAAJ0503.soap.no.null.to.dataHandler");
            throw new IllegalArgumentException("Null dataHandler argument to setDataHandler");
        }
        this.dataHandler = dataHandler;
        this.rawContent = null;
        if (AttachmentPartImpl.log.isLoggable(Level.FINE)) {
            AttachmentPartImpl.log.log(Level.FINE, "SAAJ0580.soap.set.Content-Type", new String[] { dataHandler.getContentType() });
        }
        this.setMimeHeader("Content-Type", dataHandler.getContentType());
        return;
    }
    
    public void removeAllMimeHeaders() {
        this.headers.removeAllHeaders();
    }
    
    public void removeMimeHeader(String header) {
        this.headers.removeHeader(header);
    }
    
    public String[] getMimeHeader(String name) {
        return this.headers.getHeader(name);
    }
    
    public void setMimeHeader(String name, String value) {
        this.headers.setHeader(name, value);
    }
    
    public void addMimeHeader(String name, String value) {
        this.headers.addHeader(name, value);
    }
    
    public Iterator getAllMimeHeaders() {
        return this.headers.getAllHeaders();
    }
    
    public Iterator getMatchingMimeHeaders(String[] names) {
        return this.headers.getMatchingHeaders(names);
    }
    
    public Iterator getNonMatchingMimeHeaders(String[] names) {
        return this.headers.getNonMatchingHeaders(names);
    }
    
    boolean hasAllHeaders(MimeHeaders hdrs) {
        if (hdrs != null) {
            Iterator<Object> i = hdrs.getAllHeaders();
            while (i.hasNext()) {
                MimeHeader hdr = (MimeHeader)i.next();
                String[] values = this.headers.getHeader(hdr.getName());
                boolean found = false;
                if (values != null) {
                    int j = 0;
                    while (j < values.length) {
                        if (hdr.getValue().equalsIgnoreCase(values[j])) {
                            found = true;
                            break;
                        }
                        ++j;
                    }
                }
                if (!found) {
                    return false;
                }
                continue;
            }
        }
        return true;
    }
    
    MimeBodyPart getMimePart() throws SOAPException {
        try {
            if (this.mimePart != null) {
                return new MimeBodyPart(this.mimePart);
            }
            if (this.rawContent != null) {
                copyMimeHeaders(this.headers, this.rawContent);
                return this.rawContent;
            }
            MimeBodyPart envelope = new MimeBodyPart();
            envelope.setDataHandler(this.dataHandler);
            copyMimeHeaders(this.headers, envelope);
            return envelope;
        }
        catch (Exception ex) {
            AttachmentPartImpl.log.severe("SAAJ0504.soap.cannot.externalize.attachment");
            throw new SOAPExceptionImpl("Unable to externalize attachment", ex);
        }
    }
    
    public static void copyMimeHeaders(MimeHeaders headers, MimeBodyPart mbp) throws SOAPException {
        Iterator<Object> i = headers.getAllHeaders();
        while (i.hasNext()) {
            try {
                MimeHeader mh = (MimeHeader)i.next();
                mbp.setHeader(mh.getName(), mh.getValue());
            }
            catch (Exception ex) {
                AttachmentPartImpl.log.severe("SAAJ0505.soap.cannot.copy.mime.hdr");
                throw new SOAPExceptionImpl("Unable to copy MIME header", ex);
            }
        }
    }
    
    public static void copyMimeHeaders(MimeBodyPart mbp, AttachmentPartImpl ap) throws SOAPException {
        try {
            List<Object> hdr = mbp.getAllHeaders();
            for (int sz = hdr.size(), i = 0; i < sz; ++i) {
                com.sun.xml.internal.messaging.saaj.packaging.mime.Header h = (com.sun.xml.internal.messaging.saaj.packaging.mime.Header)hdr.get(i);
                if (!h.getName().equalsIgnoreCase("Content-Type")) {
                    ap.addMimeHeader(h.getName(), h.getValue());
                }
            }
        }
        catch (Exception ex) {
            AttachmentPartImpl.log.severe("SAAJ0506.soap.cannot.copy.mime.hdrs.into.attachment");
            throw new SOAPExceptionImpl("Unable to copy MIME headers into attachment", ex);
        }
    }
    
    public void setBase64Content(InputStream content, String contentType) throws SOAPException {
        if (this.mimePart != null) {
            this.mimePart.close();
            this.mimePart = null;
        }
        this.dataHandler = null;
        InputStream decoded = null;
        try {
            decoded = MimeUtility.decode(content, "base64");
            InternetHeaders hdrs = new InternetHeaders();
            hdrs.setHeader("Content-Type", contentType);
            ByteOutputStream bos = new ByteOutputStream();
            bos.write(decoded);
            this.rawContent = new MimeBodyPart(hdrs, bos.getBytes(), bos.getCount());
            this.setMimeHeader("Content-Type", contentType);
        }
        catch (Exception e) {
            AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0578.soap.attachment.setbase64content.exception", e);
            throw new SOAPExceptionImpl(e.getLocalizedMessage());
        }
        finally {
            try {
                decoded.close();
            }
            catch (IOException ex) {
                throw new SOAPException(ex);
            }
        }
    }
    
    public InputStream getBase64Content() throws SOAPException {
        InputStream stream;
        if (this.mimePart != null) {
            stream = this.mimePart.read();
        }
        else if (this.rawContent != null) {
            try {
                stream = this.rawContent.getInputStream();
            }
            catch (Exception e2) {
                AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0579.soap.attachment.getbase64content.exception", e2);
                throw new SOAPExceptionImpl(e2.getLocalizedMessage());
            }
        }
        else if (this.dataHandler != null) {
            try {
                stream = this.dataHandler.getInputStream();
            }
            catch (IOException e) {
                AttachmentPartImpl.log.severe("SAAJ0574.soap.attachment.datahandler.ioexception");
                throw new SOAPExceptionImpl("DataHandler error" + e);
            }
        }
        else {
            AttachmentPartImpl.log.severe("SAAJ0572.soap.no.content.for.attachment");
            throw new SOAPExceptionImpl("No data handler/content associated with this attachment");
        }
        int size = 1024;
        if (stream != null) {
            try {
                ByteArrayOutputStream bos = new ByteArrayOutputStream(size);
                OutputStream ret = MimeUtility.encode(bos, "base64");
                byte[] buf = new byte[size];
                int len;
                while ((len = stream.read(buf, 0, size)) != -1) {
                    ret.write(buf, 0, len);
                }
                ret.flush();
                buf = bos.toByteArray();
                return new ByteArrayInputStream(buf);
            }
            catch (Exception e3) {
                AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0579.soap.attachment.getbase64content.exception", e3);
                throw new SOAPExceptionImpl(e3.getLocalizedMessage());
            }
            finally {
                try {
                    stream.close();
                }
                catch (IOException ex) {}
            }
        }
        AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0572.soap.no.content.for.attachment");
        throw new SOAPExceptionImpl("No data handler/content associated with this attachment");
    }
    
    public void setRawContent(InputStream content, String contentType) throws SOAPException {
        if (this.mimePart != null) {
            this.mimePart.close();
            this.mimePart = null;
        }
        this.dataHandler = null;
        try {
            InternetHeaders hdrs = new InternetHeaders();
            hdrs.setHeader("Content-Type", contentType);
            ByteOutputStream bos = new ByteOutputStream();
            bos.write(content);
            this.rawContent = new MimeBodyPart(hdrs, bos.getBytes(), bos.getCount());
            this.setMimeHeader("Content-Type", contentType);
        }
        catch (Exception e) {
            AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0576.soap.attachment.setrawcontent.exception", e);
            throw new SOAPExceptionImpl(e.getLocalizedMessage());
        }
        finally {
            try {
                content.close();
            }
            catch (IOException ex) {
                throw new SOAPException(ex);
            }
        }
    }
    
    public void setRawContentBytes(byte[] content, int off, int len, String contentType) throws SOAPException {
        if (this.mimePart != null) {
            this.mimePart.close();
            this.mimePart = null;
        }
        if (content == null) {
            throw new SOAPExceptionImpl("Null content passed to setRawContentBytes");
        }
        this.dataHandler = null;
        try {
            InternetHeaders hdrs = new InternetHeaders();
            hdrs.setHeader("Content-Type", contentType);
            this.rawContent = new MimeBodyPart(hdrs, content, off, len);
            this.setMimeHeader("Content-Type", contentType);
        }
        catch (Exception e) {
            AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0576.soap.attachment.setrawcontent.exception", e);
            throw new SOAPExceptionImpl(e.getLocalizedMessage());
        }
        return;
    }
    
    public InputStream getRawContent() throws SOAPException {
        if (this.mimePart != null) {
            return this.mimePart.read();
        }
        if (this.rawContent != null) {
            try {
                return this.rawContent.getInputStream();
            }
            catch (Exception e2) {
                AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0577.soap.attachment.getrawcontent.exception", e2);
                throw new SOAPExceptionImpl(e2.getLocalizedMessage());
            }
        }
        else if (this.dataHandler != null) {
            try {
                return this.dataHandler.getInputStream();
            }
            catch (IOException e) {
                AttachmentPartImpl.log.severe("SAAJ0574.soap.attachment.datahandler.ioexception");
                throw new SOAPExceptionImpl("DataHandler error" + e);
            }
        }
        else {
            AttachmentPartImpl.log.severe("SAAJ0572.soap.no.content.for.attachment");
            throw new SOAPExceptionImpl("No data handler/content associated with this attachment");
        }
    }
    
    public byte[] getRawContentBytes() throws SOAPException {
        if (this.mimePart != null) {
            try {
                InputStream ret = this.mimePart.read();
                return ASCIIUtility.getBytes(ret);
            }
            catch (IOException ex) {
                AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0577.soap.attachment.getrawcontent.exception", ex);
                throw new SOAPExceptionImpl(ex);
            }
        }
        else if (this.rawContent != null) {
            try {
                InputStream ret = this.rawContent.getInputStream();
                return ASCIIUtility.getBytes(ret);
            }
            catch (Exception e2) {
                AttachmentPartImpl.log.log(Level.SEVERE, "SAAJ0577.soap.attachment.getrawcontent.exception", e2);
                throw new SOAPExceptionImpl(e2);
            }
        }
        else if (this.dataHandler != null) {
            try {
                InputStream ret = this.dataHandler.getInputStream();
                return ASCIIUtility.getBytes(ret);
            }
            catch (IOException e) {
                AttachmentPartImpl.log.severe("SAAJ0574.soap.attachment.datahandler.ioexception");
                throw new SOAPExceptionImpl("DataHandler error" + e);
            }
        }
        else {
            AttachmentPartImpl.log.severe("SAAJ0572.soap.no.content.for.attachment");
            throw new SOAPExceptionImpl("No data handler/content associated with this attachment");
        }
    }
    
    public boolean equals(Object o) {
        return this == o;
    }
    
    public int hashCode() {
        return super.hashCode();
    }
    
    public MimeHeaders getMimeHeaders() {
        return this.headers;
    }
    
    public static void initializeJavaActivationHandlers() {
        try {
            CommandMap map = CommandMap.getDefaultCommandMap();
            if ((map instanceof MailcapCommandMap)) {
                MailcapCommandMap mailMap = (MailcapCommandMap)map;
                if (!cmdMapInitialized(mailMap)) {
                    mailMap.addMailcap("text/xml;;x-java-content-handler=com.sun.xml.internal.messaging.saaj.soap.XmlDataContentHandler");
                    mailMap.addMailcap("application/xml;;x-java-content-handler=com.sun.xml.internal.messaging.saaj.soap.XmlDataContentHandler");
                    mailMap.addMailcap("application/fastinfoset;;x-java-content-handler=com.sun.xml.internal.messaging.saaj.soap.FastInfosetDataContentHandler");
                    mailMap.addMailcap("image/*;;x-java-content-handler=com.sun.xml.internal.messaging.saaj.soap.ImageDataContentHandler");
                    mailMap.addMailcap("text/plain;;x-java-content-handler=com.sun.xml.internal.messaging.saaj.soap.StringDataContentHandler");
                }
            }
        }
        catch (Throwable t) {}
    }
    
    private static boolean cmdMapInitialized(MailcapCommandMap mailMap) {
        CommandInfo[] commands = mailMap.getAllCommands("application/fastinfoset");
        if (commands == null || commands.length == 0) {
            return false;
        }
        String saajClassName = "com.sun.xml.internal.ws.binding.FastInfosetDataContentHandler";
        int length = commands.length;
        int i = 0;
        while (i < length) {
            CommandInfo command = commands[i];
            String commandClass = command.getCommandClass();
            if (saajClassName.equals(commandClass)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    static {
        log = Logger.getLogger("com.sun.xml.internal.messaging.saaj.soap", "com.sun.xml.internal.messaging.saaj.soap.LocalStrings");
    }
}
